import React, { useContext } from 'react';
import { useInstance } from '@asuka/di';
import { SSRSharedContext, SSRContext, SSRStates } from './ssr/ssr-context';
import { SSRStateCacheInstance } from './ssr/ssr-states';
import { SSR_LOADED_KEY } from './ssr/constants';
import produce from 'immer';
function _useAyanamiDispatchers(ayanami) {
    const appDispatcher = React.useMemo(() => {
        const state = ayanami.state;
        const actionsCreator = ayanami.getActions();
        return Object.keys(actionsCreator).reduce((acc, cur) => {
            acc[cur] = (payload) => {
                const action = actionsCreator[cur](payload);
                state.dispatch(action);
            };
            return acc;
        }, Object.create(null));
    }, [ayanami]);
    return appDispatcher;
}
export function useAyanamiDispatchers(A) {
    const { ayanami } = _useState(A);
    return _useAyanamiDispatchers(ayanami);
}
function _useAyanamiState(state, selector, mutateStateOnFirstRendering) {
    const [appState, setState] = React.useState(() => {
        let initialState = state.getState();
        if (typeof mutateStateOnFirstRendering === 'function') {
            initialState = produce(initialState, mutateStateOnFirstRendering);
            state.state$.next(initialState);
        }
        return selector && !Reflect.getMetadata(SSR_LOADED_KEY, state)
            ? selector(initialState)
            : initialState;
    });
    const stateObserver = React.useCallback((s) => {
        if (Reflect.getMetadata(SSR_LOADED_KEY, state)) {
            Reflect.deleteMetadata(SSR_LOADED_KEY, state);
        }
        else {
            setState(selector ? selector(s) : s);
        }
    }, []);
    // https://reactjs.org/docs/hooks-faq.html#how-do-i-implement-getderivedstatefromprops
    // do not put subscribe in useEffect
    const subscribeFn = React.useCallback(() => state.subscribeState(stateObserver), [state]);
    const disposeFn = subscribeFn();
    React.useEffect(() => {
        return () => disposeFn();
    }, [state]);
    return appState;
}
export function useAyanamiState(A, config) {
    const { state } = _useState(A);
    return _useAyanamiState(state, config?.selector, config?.mutateStateOnFirstRendering);
}
export function useAyanami(A, config) {
    const { ayanami, state } = _useState(A);
    const appState = _useAyanamiState(state, config?.selector, config?.mutateStateOnFirstRendering);
    const appDispatcher = _useAyanamiDispatchers(ayanami);
    return [appState, appDispatcher];
}
function _useState(A) {
    const ssrSharedContext = useContext(SSRSharedContext);
    const ssrContext = useContext(SSRContext);
    const ayanami = useInstance(A);
    const state = React.useMemo(() => {
        return SSRStateCacheInstance.has(ssrSharedContext, A)
            ? SSRStateCacheInstance.get(ssrSharedContext, A)
            : ssrContext
                ? SSRStates.get(ssrContext)
                : ayanami.createState();
    }, [ayanami, ssrContext, ssrSharedContext]);
    return { ayanami, state };
}
